--saferun( [==[ --

import "Turbine.Gameplay";
import "Turbine.UI.Extensions";
import "Deusdictum.Plugins.WardenEase.WEQuickslotWindow";

WEController = class( Turbine.UI.Window );

function WEController:Constructor()
	Turbine.UI.Window.Constructor( self );

	self.allowedChange = 0;
	self.spacing = 2;
	self.extraSpacing = 3;
	self.quickslotSize = 36;
	self.quickslots = { };
	self.builder = { };
	self.gambitBar = { };

	self:LoadSettings();
	self:SetPosition( self.settings.left, self.settings.top );
	self:SetBackColor(Turbine.UI.Color(0, 0, 0, 0));

	self.player = Turbine.Gameplay.LocalPlayer.GetInstance();
	self.player.LevelChanged = function( sender, args )
		self:RefreshQuickslots();
	end
	
	self.menuButton = Turbine.UI.Control();
	self.menuButton:SetParent( self );
	self.menuButton:SetBackground( "Deusdictum/Resources/DownArrow.tga" );
	self.menuButton:SetSize( 14, 13 );
	
	self.menuButton.MouseDown = function( sender, args )
		local contextMenu = Turbine.UI.ContextMenu()
		local contextMenuItems = contextMenu:GetItems();
		local barType = { "Spear", "Fist", "Shield" };

		for k,v in pairs( { "Spear", "Fist", "Shield" } ) do
			local subMenu = Turbine.UI.MenuItem( v );
			subMenu:SetChecked( self.settings.enabled[v] );
			subMenu.Click = function( sender, args )
				self.settings.enabled[v] = not self.settings.enabled[v];
				if self.settings.enabled[v] then
					self.gambitBar[v] = Deusdictum.Plugins.WardenEase.WEQuickslotWindow(v, self.settings[v]);
				else
					self.gambitBar[v]:SetVisible(false);
					self.gambitBar[v]:SetParent(nil);
					self.gambitBar[v] = nil;
				end
				self:SaveSettings();
			end
			contextMenuItems:Add( subMenu );
		end

		contextMenu:ShowMenu();
	end

	self.PositionChanged = function( sender, args )
		self.settings.left, self.settings.top = self:GetPosition();
		self:SaveSettings();
	end

	for k,v in pairs(self.settings.enabled) do
		if v then
			self.gambitBar[k] = Deusdictum.Plugins.WardenEase.WEQuickslotWindow(k, self.settings[k]);
		end
	end
	
	self:RefreshQuickslots();
	self.DragBar = Deusdictum.UI.DragBar( self, "WardenEase Main Window" );
	
	self:SetVisible( true );
	self:SetOpacity( 1 );
end

function WEController:LoadSettings()
	self.settings = Turbine.PluginData.Load( Turbine.DataScope.Character, "WardenEase" );

	if ( type( self.settings ) ~= "table" ) then
		self.settings = { };
	end
	if ( type( self.settings.enabled ) ~= "table" ) then
		self.settings.enabled = { };
		self.settings.enabled["Spear"] = true;
		self.settings.enabled["Shield"] = true;
		self.settings.enabled["Fist"] = true;
	end

	local displayWidth = Turbine.UI.Display:GetWidth();
	local displayHeight = Turbine.UI.Display:GetHeight();

	if ( type( self.settings.left ) == "nil" ) then
		self.settings.left = Turbine.UI.Display:GetWidth() / 2 - 7 * (self.quickslotSize + self.spacing) / 2;
	end

	if ( type( self.settings.top ) == "nil" ) then
		self.settings.top = Turbine.UI.Display:GetHeight() * 3 / 4;
	end

	for k,v in pairs({ "Spear", "Fist", "Shield" }) do
		if ( type( self.settings[v] ) ~= "table" ) then
			self.settings[v] = { };
		end

		if ( (self.settings[v]).left == nil ) then
			self.settings[v].left = Turbine.UI.Display:GetWidth() * 2 / 3;
			if v == "Fist" then
				self.settings[v].left = self.settings[v].left + self.spacing + 32;
			end
			if v == "Shield" then
				self.settings[v].left = self.settings[v].left + 2 * (self.spacing + 32);
			end
		end

		if ( self.settings[v].top == nil ) then
			self.settings[v].top = Turbine.UI.Display:GetHeight() / 3;
			if v == "Fist" then
				self.settings[v].top = self.settings[v].top + self.spacing + 32;
			end
			if v == "Shield" then
				self.settings[v].top = self.settings[v].top + 2 * (self.spacing + 32);
			end
		end

		if self.settings[v].transposed == nil then
			self.settings[v].transposed = false;
		end
	end
end

function WEController:SaveSettings()
	pcall(function() Turbine.PluginData.Save( Turbine.DataScope.Character, "WardenEase", self.settings ) end);
end

function WEController:SetGambit( tGambitSequence )
	Deusdictum.Plugins.WardenEase.WEmain.builder = {};
	for i = 1, Deusdictum.Plugins.WardenEase.WEmain.maxGambitLength - #(tGambitSequence), 1 do
		Deusdictum.Plugins.WardenEase.WEmain.builder[i] = nil;
	end
	for i = Deusdictum.Plugins.WardenEase.WEmain.maxGambitLength - #(tGambitSequence) + 1, Deusdictum.Plugins.WardenEase.WEmain.maxGambitLength, 1 do
		Deusdictum.Plugins.WardenEase.WEmain.builder[i] = tGambitSequence[i + #(tGambitSequence) - Deusdictum.Plugins.WardenEase.WEmain.maxGambitLength];
	end
	Deusdictum.Plugins.WardenEase.WEmain:RefreshQuickslots();
end

function WEController:RefreshQuickslots()
	local i, k, v;
	local lvl = self.player:GetLevel();
	self.maxGambitLength = 0;
	self.nRecoverySkills = 0;
	self.nFastBuildSkills = 0;

	self.skills = {  };

	for k,v in pairs(Deusdictum.Data.Warden.Skills) do
		if (not (k == "Fist" or k == "Spear" or k == "Shield")) and lvl >= v.Level then
--		if (not (k == "Fist" or k == "Spear" or k == "Shield")) then
			table.insert(self.skills, v);
			if string.find(v.String, "Placeholder") ~= nil then
				self.maxGambitLength = self.maxGambitLength + 1;
			elseif string.find(v.String, "Recovery") ~= nil then
				self.nRecoverySkills = self.nRecoverySkills + 1;
			elseif v.Data ~= nil then
				if k ~= "Default" then
					self.nFastBuildSkills = self.nFastBuildSkills + 1;
				end
			end
		end
	end

	-- Remove any quickslots no longer required.
	for i = #(self.skills) + 1, #(self.quickslots), 1 do
		self.quickslots[#(self.skills) + 1]:SetVisible( false );
		self.quickslots[#(self.skills) + 1]:SetParent( nil );
		table.remove(self.quickslots, #(self.skills) + 1);
	end
	
	-- Add any new quickslots that should be created.
	for i = #(self.quickslots) + 1, #(self.skills), 1 do
		self.quickslots[i] = Turbine.UI.Lotro.Quickslot();
		self.quickslots[i]:SetParent( self );
		self.quickslots[i]:SetSize( self.quickslotSize, self.quickslotSize );
		self.quickslots[i]:SetAllowDrop( false );

		self.quickslots[i].ShortcutChanged = function( sender, args )
			if self.allowedChange == 0 then
				self:RefreshLayout();
			end
		end
	end

	self:RefreshLayout();
end

function WEController:pvar(tmp)
	Turbine.Shell.WriteLine(tmp);
end

function WEController:RefreshLayout()
	local displayWidth = Turbine.UI.Display:GetWidth();
	local displayHeight = Turbine.UI.Display:GetHeight();
	local modifier = 1;

	self.allowedChange = self.allowedChange + 1;
	self.slotsWide = math.max(self.maxGambitLength + self.nRecoverySkills + modifier, self.nFastBuildSkills);
	if self.nFastBuildSkills > 0 then
		self.slotsHigh = 2;
	else
		self.slotsHigh = 1;
	end

	local i, width, height;

	width = self.slotsWide * (self.spacing + self.quickslotSize) - self.spacing;
	if self.nRecoverySkills > 0 then
		width = width + self.extraSpacing;
	end
	height = (self.slotsHigh - 1) * (self.spacing + self.extraSpacing) + self.slotsHigh * self.quickslotSize + self.spacing + self.menuButton:GetHeight();

	self:SetSize(width, height)

	for i = 1, self.maxGambitLength, 1 do
		if self.builder[i] == nil then
			self.quickslots[i]:SetShortcut( Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Undef, "" ) );
		else
			self.quickslots[i]:SetShortcut( Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Skill, self.builder[i].Data ) );
		end
		self.quickslots[i]:SetPosition( (i - 1) * (self.quickslotSize + self.spacing), 0 );
	end

	self.quickslots[self.maxGambitLength + 1]:SetShortcut( Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Skill, Deusdictum.Data.Warden.Skills.Default.Data ) );
	self.quickslots[self.maxGambitLength + 1]:SetPosition( self.maxGambitLength * (self.quickslotSize + self.spacing), 0 );
	self.quickslots[self.maxGambitLength + 1]:SetVisible( true );

	if self.nRecoverySkills > 0 then
		self.quickslots[self.maxGambitLength + 2]:SetShortcut( Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Skill, Deusdictum.Data.Warden.Skills.Recovery.Data ) );
		self.quickslots[self.maxGambitLength + 2]:SetPosition( (self.maxGambitLength + modifier) * (self.quickslotSize + self.spacing) + self.extraSpacing, 0 );
		self.quickslots[self.maxGambitLength + 2]:SetVisible( true );
		if self.nRecoverySkills == 2 then
			self.quickslots[self.maxGambitLength + 3]:SetShortcut( Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Skill, Deusdictum.Data.Warden.Skills.QuickRecovery.Data ) );
			self.quickslots[self.maxGambitLength + 3]:SetPosition( (self.maxGambitLength + 1 + modifier) * (self.quickslotSize + self.spacing) + self.extraSpacing, 0 );
			self.quickslots[self.maxGambitLength + 3]:SetVisible( true );
		end
	end

	if self.nFastBuildSkills > 0 then
		local fbs = {
				Deusdictum.Data.Warden.Skills.SpearShield.Data
			      , Deusdictum.Data.Warden.Skills.SpearFist.Data
			      , Deusdictum.Data.Warden.Skills.ShieldSpear.Data
			      , Deusdictum.Data.Warden.Skills.ShieldFist.Data
			      , Deusdictum.Data.Warden.Skills.FistSpear.Data
			      , Deusdictum.Data.Warden.Skills.FistShield.Data
			    };

		local j, l;

		l = width / 2 - ((self.nFastBuildSkills * (self.quickslotSize + self.spacing) - self.spacing) / 2);

		for i = 1, self.nFastBuildSkills, 1 do
			j = i + self.maxGambitLength + 1 + self.nRecoverySkills;
			self.quickslots[j]:SetShortcut( Turbine.UI.Lotro.Shortcut( Turbine.UI.Lotro.ShortcutType.Skill, fbs[i] ) );
			self.quickslots[j]:SetPosition( l + (i - 1) * (self.quickslotSize + self.spacing), self.quickslotSize + self.spacing );
			self.quickslots[j]:SetVisible( true );
		end
	end

	self.menuButton:SetPosition( ( width - self.menuButton:GetWidth() ) / 2, (self.quickslotSize + self.spacing + self.extraSpacing) * self.slotsHigh - self.extraSpacing);
	self.allowedChange = self.allowedChange - 1;
end
-- ]==], "WEController");